@@ -393,6 +393,63 @@ SEXP emEN(NumericVector y, NumericMatrix gen, double R2 = 0.5, double alpha = 0.
                       Named("h2")=Va*cxx/(Va*cxx+Ve));
 }
 
+// [[Rcpp::export]]
+SEXP emML(NumericVector y, NumericMatrix gen,
+          Rcpp::Nullable<Rcpp::NumericVector> D = R_NilValue){
+  int maxit = 300;
+  double tol = 10e-8;
+  // Functions starts here
+  int p = gen.ncol();
+  int n = gen.nrow();
+  // Weights
+  bool P_WEIGHTS = FALSE;
+  NumericVector d(p);
+  if (D.isNotNull()){P_WEIGHTS = TRUE; d=D;}
+  // Beta, mu and epsilon
+  double b0, eM, ve, vb, h2, mu = mean(y), vy = var(y);
+  NumericVector b(p), e = y-mu;
+  // Marker variance
+  NumericVector xx(p), vx(p);
+  for(int i=0; i<p; i++){
+    xx[i] = sum(gen(_,i)*gen(_,i));
+    vx[i] = var(gen(_,i));}
+  double MSx = sum(vx), Lmb=MSx;
+  // Convergence control
+  NumericVector bc(p);
+  int numit = 0;
+  double cnv = 1;
+  // Loop
+  while(numit<maxit){
+    // Regression coefficients loop
+    bc = b+0;
+    for(int j=0; j<p; j++){
+      b0 = b[j];
+      if(P_WEIGHTS){
+        b[j] = (sum(gen(_,j)*e)+xx[j]*b0)/(xx[j]+Lmb/d[j]);
+      }else{
+        b[j] = (sum(gen(_,j)*e)+xx[j]*b0)/(xx[j]+Lmb);}
+      e = e-gen(_,j)*(b[j]-b0);}
+    // Variance components update
+    ve = sum(e*y)/(n-1);
+    vb = (vy-ve)/MSx;
+    Lmb = ve/vb;
+    // Intercept update
+    eM = mean(e);
+    mu = mu+eM;
+    e = e-eM;
+    // Convergence
+    ++numit;
+    cnv = sum(abs(bc-b));
+    if( cnv<tol ){break;}}
+  // Fitting the model
+  NumericVector fit(n);
+  for(int k=0; k<n; k++){ fit[k] = sum(gen(k,_)*b)+mu; }
+  h2 = vb*MSx/(vb*MSx+ve);
+  // Output
+  return List::create(Named("mu")=mu, Named("b")=b,
+                      Named("h2")=h2, Named("hat")=fit,
+                      Named("Vb")=vb, Named("Ve")=ve);}
+
 // [[Rcpp::export]]
 int calcSize(NumericVector col, NumericVector fam){
   int scol = col.size();
@@ -536,7 +593,7 @@ SEXP KMUP(NumericMatrix X, NumericVector b, NumericVector d, NumericVector xx, N
         d[j] = 1;
         e = e1;
       }else{
-        b[j] = R::rnorm(0,sqrt(Ve/(xx[j]+L[j])));
+        b[j] = 0;
         d[j] = 0;
         e = e2;
       }
@@ -592,7 +649,7 @@ SEXP KMUP2(NumericMatrix X, NumericVector Use, NumericVector b,  NumericVector d
         d[j] = 1;
         e0 = e1;
       }else{
-        b[j] = R::rnorm(0,sqrt(Ve/(xx[j]+L[j])));
+        b[j] = 0;
         d[j] = 0;
         e0 = e2;
       }
